/*
 * mondo-checksum.c 
 * Copyright 2000, Hugo Rabson <hugo@firstlinux.net> and Steve Pitts.
 * Mondo-checksum is licensed under the GNU General Public License
 * This file is part of the Mondo CD archive/recovery suite;
 * purpose is to chop the filelist into N Mb sets, and write a list
 * of really big files to a separate list
 * 
*/



#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <getopt.h>
#include <errno.h>

#define MIN_MUNCHABLE_FILESIZE 2   /* Kb; any smaller, and we shan't bother */

char *GetFileChecksum(char*);
int MakeChecksums(char*,char*,char*);
long SizeOfFile(char*);
void TrimFname(char*);
void TrimStr(char*);
int VerifyChecksums(char*,char*,char*);



char *GetFileChecksum(char *curr_fname)
{
  static char curr_cksum[1000];
  char syscall[1000],
    *sz_ptr;
  int res;
  FILE*ftmp,*pout;

  curr_cksum[0]='\0';

  if (SizeOfFile(curr_fname)<MIN_MUNCHABLE_FILESIZE)
    {
      return(curr_cksum);
    }

  // if it's a device, don't bother: just say '' :-)
  if ( (strncmp(curr_fname,"/dev/",5)==0) || (SizeOfFile(curr_fname)<=0) )
    {
      return(curr_cksum);
    }

  //  printf("Processing %s\n",curr_fname);

  sprintf(syscall,"md5sum \"%s\" 2> /dev/null",curr_fname);
  //  printf("Getting cksum of %s ...",curr_fname);

  fflush(NULL);
  pout=popen(syscall,"r");
  if (!pout)
    {
      //      printf("Can't do syscall %s\n",syscall);
      return(curr_cksum);
    }
  fgets(curr_cksum,999,pout);
  pclose(pout);
  sz_ptr=(char*)strchr(curr_cksum,' ');
  if (!sz_ptr)
    {
      //      printf("Can't get checksum for %s\n",curr_fname);
      sz_ptr=curr_cksum;
    }
  *sz_ptr='\0';
  //  printf("curr_cksum(%s) = %s\n",curr_fname,curr_cksum);
  return(curr_cksum);
}

int MakeChecksums(char *filelist, char*cksumlist, char*comppath)
{
  FILE*fin,
    *ftmp,
    *fout;
  int i,
    res;
  char stub_fname[1000],
    curr_fname[1000],
    curr_cksum[1000],
    *sz_ptr;
  fin=fopen(filelist,"r");
  if (fin==NULL) {printf("Can't open %s\n",filelist);return(1);}
  fout=fopen(cksumlist,"w");
  if (fout==NULL) {fclose(fin);printf("Can't open %s\n",cksumlist);return(1);}
  for( fgets(stub_fname,999,fin); !feof(fin); fgets(stub_fname,999,fin) )
    {
      TrimStr(stub_fname);
      sprintf(curr_fname,"%s%s",comppath,stub_fname);
      TrimFname(curr_fname);
      strcpy(curr_cksum,GetFileChecksum(curr_fname));
      fprintf(fout,"%s\t%s\n",curr_fname,curr_cksum);
    }
  fclose(fout);
  fclose(fin);
  return(0);
} /* MakeChecksums */



long SizeOfFile(char *fname)
{
  long len;
  FILE *ftemp;
  struct stat buf;

  if (strstr(fname,"/dev/")!= (int)NULL)
    {
      return(0);
    }

  if (lstat(fname,&buf)!=0)
    {
      //      printf("Unable to fstat %s\n",fname);
      return(-1);
    }

  len=buf.st_size>>10;

  return(len);
} /* SizeOfFile */





void TrimStr(char*str)
{
  int i;
  i=strlen(str);
  while(str[i-1]<32 && i>0)
    {
      str[--i]='\0';
    }
} /* TrimStr */


void TrimFname(char*str)
{
  char temp[1000];
  int i,len;

  len=strlen(str);
  for(i=0;str[i]=='/';i++);
  if (i>0); {i--;}
  strncpy(temp,str+i,999-i);
  strcpy(str,temp);
}


int VerifyChecksums(char *filelistfname, char*cksumlistfname, char*comppath)
{
  FILE*ffilelist,
    *fchecksum,
    *ftmp,
    *fstderr;
  long size_of_file;
  int i,
    res,
    files_processed,
    retval;
  char curr_fl_fname[1000],
    curr_ck_fname[1000],
    stub_fl_fname[1000],
    stub_ck_fname[1000],
    real_checksum[1000],
    recorded_checksum[1000],
    *sz_ptr;

  files_processed=0;
  retval=0;
  ffilelist=fopen(filelistfname,"r");
  if (ffilelist==NULL) {printf("Can't open %s\n",filelistfname);return(1);}
  fchecksum=fopen(cksumlistfname,"r");
  if (fchecksum==NULL) {fclose(ffilelist);printf("Can't open %s\n",cksumlistfname);return(1);}
  for( fgets(stub_fl_fname,999,ffilelist); !feof(ffilelist); fgets(stub_fl_fname,999,ffilelist) )
    {
      TrimStr(stub_fl_fname);
      sprintf(curr_fl_fname,"%s%s",comppath,stub_fl_fname);
      TrimFname(curr_fl_fname);
      fgets(stub_ck_fname,999,fchecksum);
      TrimStr(stub_ck_fname);
      sprintf(curr_ck_fname,"%s%s",comppath,stub_ck_fname);
      TrimFname(curr_ck_fname);
      //printf("Verifying %s\n",curr_fl_fname);
      //printf("curr_fl_fname = %s\ncurr_ck_fname = %s\n",curr_fl_fname,curr_ck_fname);
      sz_ptr=(char*)strchr(curr_ck_fname,'\t');
      if (!sz_ptr)
	{
	  recorded_checksum[0]='\0';
	} else {
	  *sz_ptr='\0';
	  strcpy(recorded_checksum,sz_ptr+1);
	  TrimStr(recorded_checksum);
	}

      if (strcmp(curr_ck_fname,curr_fl_fname)!=0)
	{
	  printf("Warning - %s != %s\n",curr_ck_fname,curr_fl_fname);
	  break;
	}

      size_of_file=SizeOfFile(curr_fl_fname);
      if (size_of_file < 0)
	{
	  printf("%s (not found on filesystem)\n",curr_fl_fname);
	}
      else if (size_of_file < MIN_MUNCHABLE_FILESIZE)
	{
	  //	  printf("%s (too small to worry about)\n",curr_fl_fname);
	}
      else
	{
	  strcpy(real_checksum,GetFileChecksum(curr_fl_fname));
	  if (strcmp(real_checksum,recorded_checksum)==0)
	    {
	      //    printf("%s checked out. Woohoo! :-)\n",curr_fl_fname);
	    }
	  else
	    {
	      printf("%s (is %s; should be %s)\n",stub_fl_fname,real_checksum,recorded_checksum);
	      retval++;
	    }
	}
      files_processed++;
    }
  fclose(fchecksum);
  fclose(ffilelist);
  return(retval);
} /* VerifyChecksums */



int main(int argc,char *argv[])
{
  int res;
  char filelist[1000], 
    cksumlist[1000],
    comppath[1000];

  // printf("mondo-checksum ------------------ Hugo Rabson\n");

  // argv[0] is the program's name; argv[1-4] are the params;
  // so, if we're given 4 cool parameters, argc==5 :-)
  if ( argc != 5 ) {
    printf("mondo-checksum <filelist> <cklist> <comppath> <--make|--verify>\n");
    return(1);
  }
  strncpy(filelist,argv[1],1000);
  strncpy(cksumlist,argv[2],1000);
  strncpy(comppath,argv[3],1000);
  if (strcmp(argv[4],"--make")==0)
    {
      res=MakeChecksums(filelist,cksumlist,comppath);
      return(res);
    }
  else if (strcmp(argv[4],"--verify")==0)
    {
      res=VerifyChecksums(filelist,cksumlist,comppath);
      return(res);
    }
  else
    {
      printf("Must be --make or --verify\n");
      return(1);
    }
} /* main */


